import os
import FreeCADGui as Gui
from pivy import coin

from .. import settings

class BaseViewProvider:
    ICON = None
    TYPE = None
    EXTENSIONS = []

    def __init__(self, viewprovider):
        self.viewprovider = viewprovider
        self.feature = viewprovider.Object.Proxy.feature

        self.icon = ''

        if self.ICON:
            self.icon = os.path.join(settings.ICONPATH, self.ICON)

        viewprovider.Proxy = self
        
        self.setup_extensions()

    def setup_extensions(self):
        if hasattr(self, 'EXTENSIONS'):
            for ext in self.EXTENSIONS:
                self.feature.addExtension(ext)

    def attach(self, vobj):
        self.standard = coin.SoGroup()
        vobj.addDisplayMode(self.standard, "Standard")

    def doubleClicked(self, vobj):
        Gui.activateWorkbench("KiConnect")
        Gui.Selection.clearSelection()

    def getIcon(self):
        return self.icon

    def getDisplayModes(self,obj):
        '''Return a list of display modes.'''
        return [ 'Standard' ]

    def getDefaultDisplayMode(self):
        '''Return the name of the default display mode. It must be defined in getDisplayModes.'''
        return 'Standard'

    def __getstate__(self):
        return {
            'icon': self.icon
        }

    def __setstate__(self, props):
        for prop in props:
            setattr(self, prop, props[prop])
